import obs, uasyncio, config
from modbus import modbus
from sensor_drv import SensorDrv
import scheduler

class modbus_spectrolyser(SensorDrv):
    def __init__(self, serial, conf):
        SensorDrv.__init__(self)
        self.__syslog = obs.system_log('modbus_spectrolyser')
        self.modbus_obj = modbus(serial, conf)
        self.__serial = serial
        self.wait_time = config.get_config_or_default(conf, 'measurement_wait_time', 25) * 1000
        self.__sreader = uasyncio.StreamReader(self.__serial)
        self.__response_timeout_ms = 1000
        self.retry_nr = 5

    async def read_values(self):
        #Start by sending a measurement request, then wait the 'measurement_wait_time' that is defined in config 
        await self.measurement_request()
        #read register as usual, retry up to five times. If no response the driver managers initiates this function again up to the response_timeout.
        for _ in range(self.retry_nr):
            if await self.modbus_obj.read_values() == True:
                self.data = self.modbus_obj.data
                return True
            await uasyncio.sleep_ms(1000)
        return False

    async def detect_parameters(self, tag_number = ''):
        await self.measurement_request()
        for _ in range(self.retry_nr):
            if await self.modbus_obj.detect_parameters(tag_number):
                self.parameters = self.modbus_obj.parameters
                return True
            await uasyncio.sleep_ms(1000)
        return False

    async def measurement_request(self):
        #Write register to start the measurement
        # Spectrolyser: Device address (0x04), Function code (0x06 = 'Preset Single Register', Register address (0x0449), data (0x0001), CRC (0x98B9)
        for _ in range(self.retry_nr):
            self.__serial.write(b'\x04\x06\x04\x49\x00\x01\x98\xB9')
            result = await self._serial_read()
            if result != bytearray(b'\x04\x06\x04I\x00\x01\x98\xb9'):
                self.__syslog.info("Error attempt {}, unexpected response from sensor received: {}".format(_, result))
                await uasyncio.sleep_ms(1000)
            else:
                self.__syslog.info('Transmitted the measurement command, wait time: {} sec.'.format(self.wait_time/1000))
                break

        #Wait for self.wait_time ms for the sensor to obtain values
        await uasyncio.sleep_ms(self.wait_time)

    async def flush(self):
        #Not required
        pass
    

    async def _serial_read(self):
        response_timeout = obs.millis()
        response = bytearray()

        #Wait for data to appear on the bus
        while(self.__serial.any() == 0):
            if obs.elapsed_millis(response_timeout) > self.__response_timeout_ms:
                break
            uasyncio.sleep_ms(1)

        #Handle the data   
        while(self.__serial.any() != 0):
            chars_in_buffer = self.__serial.any()
            response.extend(await self.__sreader.read(chars_in_buffer))
            if obs.elapsed_millis(response_timeout) > self.__response_timeout_ms:
                break
            await uasyncio.sleep_ms(10)
        return response
